/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
package net.smartlab.web.registry;

import org.apache.commons.lang.StringUtils;

import net.smartlab.web.BusinessObject;

/**
 * The representation of a generic address. Subclasses must add the properties
 * needed to fully and correctly describe the specific address type.
 * 
 * @author rlogiacco
 * @hibernate.class schema="registry" table="`address`"
 * @uml.dependency   supplier="net.smartlab.web.registry.Entity"
 */
 
public abstract class Address extends BusinessObject implements Comparable {

	private final static long serialVersionUID = -4838144615188719235L;

	/**
	 * The unique identifier.
	 */
	private long id;

	/**
	 * The text displayed to describe this address.
	 */
	private String display;

	/**
	 * The priority assigned to this address. Addresses are grouped by type and
	 * used in descending priority order.
	 */
	private int priority;

	/** 
	 * The registry entity which references this address.
	 * @clientCardinality 0..*
	 * @supplierCardinality 1
	 * @uml.property name="entity"
	 * @uml.associationEnd inverse="addresses:net.smartlab.web.registry.Entity"
	 * @uml.association name="has "
	 */
	private Entity entity;


	/**
	 * Returns the unique identification number.
	 * 
	 * @return the unique identification number.
	 * @hibernate.id column="`id`" unsaved-value="0" generator-class="native"
	 */
	public long getId() {
		return id;
	}

	/**
	 * Sets the unique identification number.
	 * 
	 * @param id the unique identification number.
	 */
	public void setId(long id) {
		this.id = id;
	}

	/**
	 * Returns the text displayed to describe this address.
	 * 
	 * @return the text displayed to describe this address.
	 * @hibernate.property column="`display`" length="150"
	 */
	public String getDisplay() {
		return display;
	}

	/**
	 * Sets the text displayed to describe this address.
	 * 
	 * @param display the text displayed to describe this address.
	 */
	public void setDisplay(String display) {
		this.display = display;
	}

	/**
	 * Returns the priority assigned to this address. Addresses are grouped by
	 * type and used in descending priority order.
	 * 
	 * @return the priority assigned to this address
	 * @hibernate.property column="`priority`"
	 */
	public int getPriority() {
		return priority;
	}

	/**
	 * Sets the priority assigned to this address. Addresses are grouped by type
	 * and used in descending priority order.
	 * 
	 * @param priority the priority assigned to this address
	 */
	public void setPriority(int priority) {
		this.priority = priority;
	}

	/** 
	 * Returns the registry entity which references this address.
	 * @return returns the registry entity which references this address.
	 * @hibernate.many-to-one column="`entity`"
	 * @see also Entity
	 * @uml.property  name="entity"
	 */
	public Entity getEntity() {
		return entity;
	}

	/** 
	 * Sets the registry entity which references this address.
	 * @param entity the registry entity which references this address to set.
	 * @uml.property  name="entity"
	 */
	public void setEntity(Entity entity) {
		this.entity = entity;
	}

	/**
	 * Each address is characterized by owner fields 
	 * @return a String which describe the address. 
	 */
	public abstract String getFormattedString();

	/**
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(Object object) {
		Address address = ((Address)object);
		if(this.getPriority() - address.getPriority() == 0){
			return (normalizeNull(this.getDisplay()) + normalizeNull(this.getFormattedString()))
			.compareTo(normalizeNull(address.getDisplay()) + normalizeNull(address.getFormattedString()));
		}
		return this.getPriority() - address.getPriority();
	}
	/**
	 * @see net.smartlab.web.BusinessObject#getVersion()
	 * @hibernate.version
	 */
	public long getVersion() {
		return super.getVersion();
	}
	protected String normalizeNull(String withNull){
		if(StringUtils.isEmpty(withNull)){
			return "";
		}
		return withNull;
	}
}
