/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */

package net.smartlab.web.registry;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

/**
 * Represents a corporation entity in the registry. This class could be used to
 * store commercial business entities, non-profit organization informations or
 * public corporations like countries or states, eventually through subclassing.
 * 
 * @author rlogiacco
 * @hibernate.joined-subclass schema="registry" table="`corporation`"
 * @hibernate.joined-subclass-key column="`id`"
 */
public class Corporation extends Entity {

	private static final long serialVersionUID = 2400674709089161829L;

	/** 
	 * The main address of the corporation, usually defined as <i>legal situs
	 * </i>.
	 * @directed true
	 * @uml.property name="legalSitus"
	 * @uml.associationEnd aggregation="shared" inverse="corporation:net.smartlab.web.registry.MailAddress"
	 * @uml.association name="reachable by"
	 */
	private MailAddress legalSitus;

	/**
	 * Identifing code usually used as unique financial reference to the
	 * corporation.
	 */
	private String idCode;

	/**
	 * Default Constructor
	 */
	public Corporation() {
		super();
	}

	/**
	 * Constructor by idCode
	 * 
	 * @param idCode
	 */
	public Corporation(String idCode) {
		super();
		this.idCode = idCode;
	}

	/**
	 * Constructor using fields
	 * 
	 * @param idCode
	 *            The idCode to be set
	 * @param legalSitus
	 *            The legalSitus to be set
	 */
	public Corporation(String idCode, MailAddress legalSitus) {
		super();
		this.idCode = idCode;
		setLegalSitus(legalSitus);
	}

	/** 
	 * Returns the main address of the corporation, usually defined as <i>legal
	 * situs </i>.
	 * @return the main address of the corporation.
	 * @hibernate.many-to-one column="`legalSitus`" cascade = "all"
	 *  inverse="true"
	 * @uml.property  name="legalSitus"
	 */
	public MailAddress getLegalSitus() 
	{
		return legalSitus;
	}

	/** 
	 * Setter of the property <tt>legalSitus</tt>
	 * @param legalSitus The legalSitus to set.
	 * @uml.property  name="legalSitus"
	 */
	public void setLegalSitus(MailAddress legalSitus) 
	{
		this.legalSitus = legalSitus;
	}

	/**
	 * Returns the identifing code usually used as unique financial reference to
	 * the corporation.
	 * 
	 * @return returns the identifing code.
	 * @hibernate.property column="`idCode`"
	 */
	public String getIdCode() {
		return idCode;
	}

	/**
	 * Sets the identifing code usually used as unique financial reference to
	 * the corporation.
	 * 
	 * @param idCode
	 *            the identifing code.
	 */
	public void setIdCode(String idCode) {
		this.idCode = idCode;
	}

	/**
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object object) {
		if (!(object instanceof Corporation)) {
			return false;
		}
		Corporation rhs = (Corporation) object;
		return new EqualsBuilder().append(this.getId(), rhs.getId()).append(this.getIdCode(), rhs.getIdCode()).append(this.getLegalSitus(), rhs.getLegalSitus()).isEquals();

	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return new HashCodeBuilder(1816801317, 357614045).append(this.idCode).append(this.legalSitus).toHashCode();
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return new ToStringBuilder(this, ToStringStyle.SHORT_PREFIX_STYLE).append("IdCode", idCode).append(this.legalSitus).toString();
	}
}
