/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
 
package net.smartlab.web.registry;

import java.util.Collections;
import java.util.Comparator;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

import net.smartlab.web.StringEnumeration;

/**
 * Enumerates all the countries using their ISO codes as unique identifier.
 * 
 * @author rlogiacco
 */
public class Country extends StringEnumeration {

    private final static long serialVersionUID = 1244023731198429489L;
    
    /**
     * TODO documentation
     */
    public final static Country UNDEFINED = new Country() {

        private static final long serialVersionUID = -5816240524773472797L;

        /**
         * @see net.smartlab.web.StringEnumeration#getCode()
         */
        public String getCode() {
            return "##";
        }
        
        /**
         * @see net.smartlab.web.registry.Country#getDisplay()
         */
        public String getDisplay() {
            return "N/A";
        }

        /**
         * @see net.smartlab.web.registry.Country#getDisplay(java.util.Locale)
         */
        public String getDisplay(Locale locale) {
            return "N/A";
        }
        
    };

    private final static Set list;

    /**
     * The full list of ISO codes known to Java.
     */
    private final static Map locales = new TreeMap();
    static {
        Locale[] availables = Locale.getAvailableLocales();
        for (int i = 0; i < availables.length; i++) {
            locales.put(availables[i].getCountry().toLowerCase(), availables[i]);
        }
        TreeSet ordered = new TreeSet(new Comparator() {

            public int compare(Object first, Object second) {
                return ((Locale)first).getDisplayCountry().compareTo(((Locale)second).getDisplayCountry());
            }});
        ordered.addAll(locales.values());
        list = Collections.unmodifiableSortedSet(ordered);
    }


    /**
     * Default constructor.
     */
    public Country() {
        super();
        this.code = "N/A";
    }

    /**
     * Internal constructor.
     * 
     * @param code the ISO code used to identify the country.
     */
    protected Country(String code) {
        super.code = code.toLowerCase();
    }

    /**
     * @see net.smartlab.web.StringEnumeration#getDisplay()
     */
    public String getDisplay() {
        return ((Locale)locales.get(code)) != null ? ((Locale)locales.get(code)).getDisplayCountry() :"N/A";
    }

    /**
     * Returns the display name of this country in its localized version.
     * 
     * @param locale the language to be used to localize the country display
     *        name.
     * @return the display name of this country in its localized version.
     */
    public String getDisplay(Locale locale) {
        return ((Locale)locales.get(code)).getDisplayCountry(locale);
    }

    /**
     * @see net.smartlab.web.StringEnumeration#decode(java.lang.String)
     */
    public StringEnumeration decode(String code) {
        if (code != null && locales.get(code.toLowerCase()) != null) {
            return new Country(code);
        } else {
            return UNDEFINED;
        }
    }

    /**
     * Return the list of known countries as instances of <code>Country</code> class.
     * @return the list of known countries as instances of <code>Country</code> class.
     */
    public static Set getCountries() {
        return list;
    }

	public boolean equals(Object obj) {
		if(obj instanceof Country)
		{
			Country country = (Country)obj;
			return getDisplay() == country.getDisplay(); 
		}
		return false;
	}
    
}