/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
package net.smartlab.web.registry;

import java.util.HashSet;
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.Set;
import java.util.TreeSet;

/**
 * Represents the atomic and concrete element of the registry. Properties of
 * this class could be used to generically describe both a person and a corporation
 * and should be as flexible to allow any other possible definition.
 * <p>
 * An Entity doesn't need to provide a complete set of explicit information.  
 * However you can join a set of information between two or more <tt>Entities</tt>
 * through its related <tt>Relationships</tt>.
 * 
 * 
 * @author rlogiacco
 * @author gcontartese
 * 
 * @see net.smartlab.web.registry.Address
 * @see net.smartlab.web.registry.Relationship
 * @hibernate.joined-subclass schema="registry" table="`entity`"
 * @hibernate.joined-subclass-key column="`id`"
 */
public class Entity extends Entry {

	private final static long serialVersionUID = -8020976767994142730L;

	/** 
	 * The addresses associated to this entity.
	 * @link aggregation <{Address}>
	 * @directed directed
	 * @supplierCardinality 0..*
	 * @uml.property name="addresses"
	 * @uml.associationEnd multiplicity="(0 -1)" inverse="entity:net.smartlab.web.registry.Address"
	 * @uml.association name="has "
	 */
	protected Set addresses = new TreeSet();

	/**
	 * The relationships defined between this entity and the others defined into
	 * the registry. Relationships navigation is <b>not </b> bi-directional so
	 * any bi-directional relationship should be defined twice. Relationships
	 * through multiple entities should be defined as one-to-many where the
	 * <i>many </i> part could be difined with a common prefix used for
	 * searches.
	 * 
	 * @directed directed
	 * @label relationships
	 * @uml.property name="relationships"
	 * @uml.associationEnd multiplicity="(0 -1)" inverse="source:net.smartlab.web.registry.Relationship"
	 * @link aggregation <{Entity}>
	 * @supplierCardinality 0..*
	 */
	protected Set relationships = new HashSet();


	/**
     * Returns the <code>Address</code> at the specified position in this set.
     *
     * @param  index index of element to return.
     * @return the element at the specified position in this list.
     * @throws    IndexOutOfBoundsException if the address will not be found..
     */
	public Address getAddress(int index) {
		Iterator addresses = this.addresses.iterator();
		for (int i = 0; addresses.hasNext(); i++) {
			if (i == index) {
				return (Address)addresses.next();
			}
		}
		throw new IndexOutOfBoundsException("No address at position " + index);
	}

	/**
     * Returns the <code>Address</code> instance of specified class type
     * at the specified position in this set.
     *
     * @param  index index of element to return.
     * @param  type Class of the element to return.
     * @return the element at the specified position in this list which is istance of
     * specified class.
     * @throws    IndexOutOfBoundsException if the address will not be found..
     */
	public Address getAddress(int index, Class type) {
		Iterator addresses = this.getAddresses(type).iterator();
		for (int i = 0; addresses.hasNext(); i++) {
			if (i == index) {
				return (Address)addresses.next();
			}
			addresses.next();
		}
		throw new IndexOutOfBoundsException("No address at position " + index + " for " + type + " addresses");
	}

	/**
	 * Return the first Address present in the Set.
	 * 
	 * @return <code>Address</code> the address 
	 */
	public Address getAddress() {
		if(addresses!=null)
			return (Address)addresses.iterator().next();
		return null;
	}

	/**
	 * Returns the first address of the specified type associated to this
	 * entity.
	 * 
	 * @param type the address type in the form of a subclass of
	 *        <code>Address</code>.
	 * @return the first addresses of the specified type associated to this
	 *         entity.
	 * @throws NoSuchElementException when no address of given type was present
	 */
	public Address getAddress(Class type) throws NoSuchElementException {
		return (Address)this.getAddresses(type).iterator().next();
	}
	/**
	 * Returns the addresses of the specified type associated to this entity.
	 * 
	 * @param type the address type in the form of a subclass of
	 *        <code>Address</code>.
	 * @return the addresses of the specified type associated to this entity.
	 */
	public Set getAddresses(Class type) {
		Set result = new TreeSet();
		Iterator addresses = this.addresses.iterator();
		while (addresses.hasNext()) {
			Address address = (Address)addresses.next();
			if (type.isInstance(address)) {
				result.add(address);
			}
		}
		return result;
	}
	/** 
	 * Returns all the addresses associated with this entity.
	 * @return all the addresses associated to this entity.
	 * @hibernate.set lazy="true" cascade="all"
	 *  order-by="`priority` desc"
	 * @hibernate.collection-key column="`entity`"
	 * @hibernate.collection-one-to-many class="net.smartlab.web.registry.Address"
	 * @uml.property  name="addresses"
	 */
	public Set getAddresses() {
		return addresses;
	}

	/** 
	 * Sets the addresses associated to this entity.
	 * @param addresses the addresses associated to this entity.
	 * @uml.property  name="addresses"
	 */
	public void setAddresses(Set addresses) {
		this.addresses = addresses;
	}

	/**
	 * Adds an address to the addresses associated to this entity.
	 * 
	 * @param address the addresses to add.
	 */
	public void addAddress(Address address) {
		if(address == null)
			throw new IllegalArgumentException("Address cannot be null");
		address.setEntity(this);
		this.getAddresses().add(address);
	}

	/**
	 * Removes an address from the addresses associated to this entity.
	 * 
	 * @param address the addresses to remove.
	 */
	public void removeAddress(Address address) {
		this.addresses.remove(address);
	}

	/**
	 * Returns the relationships defined between this entity and the others
	 * defined into the registry. Relationships navigation is <b>not </b>
	 * bi-directional so any bi-directional relationship should be defined
	 * twice. Relationships through multiple entities could be defined too as no
	 * constraint is imposed to the relationship name.
	 * 
	 * @return returns the relationships.
	 * @hibernate.set lazy="true" schema="registry" table="`relationship`"
	 *                cascade="all"
	 * @hibernate.collection-key column="`source`"
	 * @hibernate.collection-composite-element class="net.smartlab.web.registry.Relationship"
	 * @uml.property  name="relationships"
	 */
	public Set getRelationships() {
		return relationships;
	}

	/**
	 * Finds the named relationships through the relationships defined between
	 * this entity and the others defined into the registry.
	 * 
	 * @param name the relationship name.
	 * @return the relationships matching the specified name.
	 */
	public Set getRelationships(String name) {
		Set result = new HashSet();
		Iterator relationships = this.relationships.iterator();
		while (relationships.hasNext()) {
			Relationship relationship = (Relationship)relationships.next();
			if (name.equals(relationship.getName())) {
				result.add(relationship);
			}
		}
		return result;
	}

	/**
	 * Sets the relationships defined between this entity and the others defined
	 * into the registry. Relationships navigation is <b>not </b> bi-directional
	 * so any bi-directional relationship should be defined twice. Relationships
	 * through multiple entities could be defined too as no constraint is
	 * imposed to the relationship name.
	 * 
	 * @param addresses the relationships to set.
	 * @uml.property  name="relationships"
	 */
	public void setRelationships(Set addresses) {
		this.relationships = addresses;
	}

	/**
	 * Establish a relationship between the current entry and another using a
	 * name as description.
	 * 
	 * @param entity the relationship target.
	 * @param name the relationship name.
	 */
	public void addRelationship(Entity entity, String name) {

		this.relationships.add(new Relationship(entity, name));

	}

	/**
	 * Drops a relationship between the current entry and the specified one
	 * distinguishing by the optional relationship name. If the relationship
	 * name is left unspecified (a<code>null</code> value) any relationship
	 * with the specified entry will be dropped.
	 * 
	 * @param entity the relationship target.
	 * @param name the optional relationship name.
	 */
	public void removeRelationship(Entity entity, String name) {
		if (name == null) {
			Iterator relationships = this.relationships.iterator();
			while (relationships.hasNext()) {
				Relationship relationship = (Relationship)relationships.next();
				if (entity.equals(relationship.getTarget())) {
					relationships.remove();
				}
			}
		} else {

			this.relationships.remove(new Relationship(entity, name));

		}
	}
	/**
	 * Drop all the existing relationships between the current entity and the one
	 * specified.
	 * @param entity The relationship target
	 */
	public void removeAllRelationships(Entity entity){
		this.removeRelationship(entity,null);
	}
}
