/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
package net.smartlab.web.registry;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import net.smartlab.web.BusinessObject;

/**
 * Represents a logical entry in the registry. Logical entries are the registry
 * elements thus they does not provide any specific information about the real
 * entry information. As consequence they are not instantiable by their own.
 * Registry entries could be categorized into two main types:
 * <ul>
 * <li>groups of entries, also known as <b>list</b>;</li>
 * <li>simple entries, also known as <b>entity</b>.</li>
 * </ul>
 * <blockquote>The categorization, as defined above, doesn't preclude the
 * definition of a list containing both entities and lists. </blockquote>
 * 
 * @author rlogiacco
 * @hibernate.class schema="registry" table="`entry`"
 */
public abstract class Entry extends BusinessObject implements Comparable {

	private final static long serialVersionUID = -2207896799625759184L;

	/**
	 * The unique identifier.
	 */
	private long id;

	/**
	 * The text displayed to describe this entry.
	 */
	private String display;

	/**
	 * An associative map of properties related to this entry. This map should
	 * be used to enrich the entry with custom values. Be carefull as search on
	 * a specific combination of properties <b>is not supported </b> by default
	 * as it could be really expensive.
	 */
	protected Map properties = new HashMap();

	/**
	 * The list set in witch the entry is contained
	 */
	private Set lists = new TreeSet();


	/**
	 * Returns the unique identifier.
	 * 
	 * @return the unique identifier.
	 * @hibernate.id column="`id`" unsaved-value="0" generator-class="native"
	 */
	public long getId() {
		return this.id;
	}

	/**
	 * Sets the unique identifier.
	 * 
	 * @param id the unique identifier to set.
	 */
	public void setId(long id) {
		this.id = id;
	}

	/**
	 * Returns the text displayed to describe this entry.
	 * 
	 * @return the text displayed to describe this entry.
	 * @hibernate.property column="`display`" length="50" not-null="true"
	 */
	public String getDisplay() {
		return this.display;
	}

	/**
	 * Sets the text displayed to describe this entry.
	 * 
	 * @param display the text displayed to describe this entry.
	 */
	public void setDisplay(String display) {
		this.display = display;
	}

	/**
	 * Returns an associative map of properties related to this entry. This map
	 * should be used to enrich the entry with custom values. Be carefull as
	 * search on a specific combination of properties <b>is not supported </b>
	 * by default as it could be really expensive.
	 * 
	 * @return the properties associated with this entry.
	 * @hibernate.map lazy="true" schema="registry" table="`property`"
	 *                inverse="false" cascade="all"
	 * @hibernate.collection-key column="`entry`"
	 * @hibernate.collection-index column="`name`" type="java.lang.String"
	 *                             length="50"
	 * @hibernate.collection-element column="`value`" type="java.lang.String"
	 *                               length="255"
	 */
	public Map getProperties() {
		return properties;
	}

	/**
	 * Sets an associative map of properties related to this entry. This map
	 * should be used to enrich the entry with custom values. Be carefull as
	 * search on a specific combination of properties <b>is not supported </b>
	 * by default as it could be really expensive.
	 * 
	 * @param properties the properties to associate to this entry.
	 */
	protected void setProperties(Map properties) {
		this.properties = properties;
	}

	/**
	 * Sets a property into the associative map of properties related to this
	 * entry. This map should be used to enrich the entry with custom values. Be
	 * carefull as search on a specific combination of properties <b>is not
	 * supported </b> by default as it could be really expensive.
	 * 
	 * @param name the property name.
	 * @param value the property value.
	 */
	public void setProperty(String name, String value) {
		this.properties.put(name, value);
	}

	/**
	 * Removes a property from the associative map of properties related to this
	 * entry. This map should be used to enrich the entry with custom values. Be
	 * carefull as search on a specific combination of properties <b>is not
	 * supported </b> by default as it could be really expensive.
	 * 
	 * @param name the property name.
	 */
	public void removeProperty(String name) {
		this.properties.remove(name);
	}

	/**
	 * Finds a property into the associative map of properties related to this
	 * entry. This map should be used to enrich the entry with custom values. Be
	 * carefull as search on a specific combination of properties <b>is not
	 * supported </b> by default as it could be really expensive.
	 * 
	 * @param name the property name.
	 * @return the property value or <code>null</code> if a property with that
	 *         name was never defined.
	 */
	public String getProperty(String name) {
		return (String)properties.get(name);
	}

	/**
	 * @TODO documentation
	 * @return
	 * @hibernate.set lazy="true" schema="registry" table="`list_entry`"
	 *                sort="natural" inverse = "true" *
	 * @hibernate.collection-key column="`entry`"
	 * @hibernate.collection-many-to-many column="`list`"
	 *                                    class="net.smartlab.web.registry.List"
	 */
	public Set getLists() {
		return lists;
	}

	/**
	 * TODO documentation
	 * 
	 * @param lists
	 */
	public void setLists(Set lists) {
		this.lists = lists;
	}

	/**
	 * Checks if the entry represents a group of entries.
	 * 
	 * @return <code>true</code> if and only if the entry is a collection of
	 *         entries.
	 */
	public boolean isList() {
		return this instanceof List;
	}

	/**
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(Object entry) {
		if(entry == null){
			throw new IllegalArgumentException(" Entry cannot be null");
		}
		if(((Entry)entry).getDisplay() == null){
			throw new IllegalArgumentException(" Display cannot be null");
		}
		if (entry instanceof Entry) {
			return this.getDisplay().compareTo(((Entry)entry).getDisplay());
		} else {
			throw new ClassCastException();
		}
	}
	
	/**
	 * TODO documentation
	 *
	 * @param type
	 * @return
	 */
	protected boolean isInstance(Class type){
		return type.isInstance(this) ;
	}
	/**
	 * @see net.smartlab.web.BusinessObject#getVersion()
	 * @hibernate.version
	 */
	public long getVersion() {
		return super.getVersion();
	}
}
