/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
package net.smartlab.web.registry;

import java.util.Iterator;
import java.util.Set;
import java.util.TreeSet;

/**
 * This class represents a persistent collection of non duplicated entries usable whenever an entry is required.
 * 
 * @author rlogiacco
 * @hibernate.joined-subclass schema="registry" table="`list`"
 * @hibernate.joined-subclass-key column="`id`"
 */
public class List extends Entry {

	private final static long serialVersionUID = 4149872229997965218L;

	private String description;

	private int size;

	/**
	 * Stores the entries avoiding duplicates.
	 *
	 * @link aggregation <{Entry}>
	 * @directed directed
	 * @supplierCardinality 0..*
	 * @uml.property  name="entries"
	 * @uml.associationEnd  multiplicity="(0 -1)" ordering="true" inverse="set:net.smartlab.web.registry.Entry"
	 */
	private Set entries = new TreeSet();


	/**
	 * Returns the list description.
	 * 
	 * @return returns the list <code>description</code>.
	 * @hibernate.property column="description" type="text"
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * Sets the list description.
	 * 
	 * @param description the list description to set.
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * @return returns the list entries.
	 * @hibernate.set lazy="true" schema="registry" table="`list_entry`"
	 *                sort="natural"
	 * @hibernate.collection-key column="`list`"
	 * @hibernate.collection-many-to-many column="`entry`"
	 *                                    class="net.smartlab.web.registry.Entry"
	 * @uml.property  name="entries"
	 */
	public Set getEntries() {
		return entries;
	}

	/**
	 * Returns the set of entries belonging to this group of the specified type
	 * @param type
	 * @return
	 */
	public Set getEntries(Class type) {
		Set result = new TreeSet();
		Iterator iter = this.getEntries().iterator();
		while (iter.hasNext()) {
			try {
				Entry obj = (Entry)iter.next();
				if (obj.isInstance(type)) {
					result.add(obj);
				}
			} catch (Exception e) {
				// DO NOTHING
				e.printStackTrace();
			}
		}
		return result;
	}

	/**
	 * Sets the list entries.
	 * 
	 * @param entries the entries to set.
	 * @uml.property  name="entries"
	 */
	protected void setEntries(Set entries) {
		this.entries = entries;
	}

	/**
	 * Ensures the list has no entries.
	 */
	public void clearEntries() {
		entries.clear();
	}

	/**
	 * Adds an entry to this list.
	 *
	 * @param entry the entry to add.
	 */
	public void add(Entry entry) {
		this.getEntries().add(entry);
	}

	/**
	 * Removes an entry from this list.
	 * 
	 * @param entry the entry to remove.
	 */
	public void remove(Entry entry) {
		this.getEntries().remove(entry);
	}

	/**
	 * Returns the number of entries belonging to this list.
	 * 
	 * @return the number of entries belonging to this list.
	 * @hibernate.property insert="false" update="false" formula="(select count(*) from registry.list_entry where registry.list_entry.list=id )"
	 */
	public int getSize() {
		return this.size;
	}

	/**
	 * Sets the size.
	 * 
	 * @param size the size to set
	 */
	public void setSize(int size) {
		this.size = size;
	}
}
