/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
package net.smartlab.web.registry;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

/**
 * Represents a fully qualified mail address usable for postal delivery
 * 
 * @author rlogiacco
 * @hibernate.joined-subclass schema="registry" table="`mail-address`"
 * @hibernate.joined-subclass-key column="`id`"
 */
public class MailAddress extends Address {

	private final static long serialVersionUID = -678094002234942516L;

	/**
	 * An optional description for delivery purposes.
	 */
	private String at;

	/**
	 * The fully qualified city.
	 * 
	 * @directed true
	 * @stereotype component 
	 * @uml.property name="locality"
	 * @uml.associationEnd multiplicity="(1 1)" aggregation="composite" inverse="mailAddress:net.smartlab.web.registry.Locality"
	 */
	private Locality locality = new Locality();

	/**
	 * An optional name to describe an additional subdivision of the city.
	 */
	private String hamlet;

	/**
	 * The delivery code used in local area identifications.
	 */
	private String zipCode;

	/**
	 * The street.
	 */
	private String street;

	/**
	 * The string identifing the street subdivision.
	 */
	private String civic;


	/**
	 * Default constructor
	 */
	public MailAddress() {
		super();
	}
	/**
	 * Constructor using fields
	 * @param street the street to be set
	 * @param civic the civic to be set
	 */
	public MailAddress(String street, String civic) {
		this(street, civic, null, null, null, null);
	}

	/**
	 * Constructor using fields
	 * @param street the street to be set
	 * @param civic the civic to be set
	 * @param locality the locality to be set
	 */
	public MailAddress(String street, String civic, Locality locality) {
		this(street, civic, null, locality, null, null);
	}

	/**
	 * Constructor using all fields
	 * 
	 * @param street the street to be set
	 * @param civic the civic to be set
	 * @param at The at to be set
	 * @param locality the locality to be set
	 * @param hamlet the hamlet to be set
	 * @param zipCode the zipCode to be set
	 * 
	 */
	public MailAddress(String street, String civic, String at, Locality locality, String hamlet, String zipCode) {
		super();
		this.at = at;
		this.locality = locality == null ? new Locality() : locality;
		this.hamlet = hamlet;
		this.zipCode = zipCode;
		this.street = street;
		this.civic = civic;
	}

	/**
	 * Returns the at 
	 * @return returns the at.
	 * @hibernate.property column="`at`" length="250"
	 */
	public String getAt() {
		return at;
	}

	/**
	 * Sets the at 
	 * 
	 * @param at the at to set.
	 */
	public void setAt(String at) {
		this.at = at;
	}

	/**
	 * Returns the civic
	 * 
	 * @return returns the civic.
	 * @hibernate.property column="`civic`" length="50"
	 */
	public String getCivic() {
		return civic;
	}

	/**
	 * Sets the civic
	 * 
	 * @param civic the civic to set.
	 */
	public void setCivic(String civic) {
		this.civic = civic;
	}

	/**
	 * Returns the locality
	 * 
	 * @hibernate.component
	 * @uml.property  name="locality"
	 * @return returns the locality.
	 */
	public Locality getLocality() {
		return locality;
	}

	/**
	 * Sets the locality
	 * @uml.property  name="locality"
	 * @param locality the locality to set.
	 */
	public void setLocality(Locality locality) {
		this.locality = locality;
	}

	/**
	 * Returns the hamlet
	 * 
	 * @return returns the hamlet.
	 * @hibernate.property column="`hamlet`" length="250"
	 */
	public String getHamlet() {
		return hamlet;
	}

	/**
	 * Sets the hamlet
	 * 
	 * @param hamlet the hamlet to set.
	 */
	public void setHamlet(String hamlet) {
		this.hamlet = hamlet;
	}

	/**
	 * Returns the street
	 * 
	 * @return returns the street.
	 * @hibernate.property column="`street`" length="250"
	 */
	public String getStreet() {
		return street;
	}

	/**
	 * Set the street
	 * 
	 * @param street the street to set.
	 */
	public void setStreet(String street) {
		this.street = street;
	}

	/**
	 * Returns the zip code
	 * 
	 * @return returns the zipCode.
	 * @hibernate.property column="`zipCode`" length="25"
	 */
	public String getZipCode() {
		return zipCode;
	}

	/**
	 * Returns the zip code
	 * 
	 * @param zipCode the zipCode to set.
	 */
	public void setZipCode(String zipCode) {
		this.zipCode = zipCode;
	}

	/**
	 * @see net.smartlab.web.registry.Address#getFormattedString()
	 */
	public String getFormattedString() {
		StringBuffer address = new StringBuffer();
		address.append(this.street != null ? this.street + " " : "");
		address.append(this.civic != null ? this.civic + " " : "");
		address.append(this.zipCode != null ? this.zipCode + " " : "");
		if (this.locality != null) {
			address.append(this.locality.getCity() != null ? this.locality.getCity() + " " : "");
			address.append(this.locality.getState() != null ? this.locality.getState() + " " : "");
			address.append(this.locality.getCountry() != null ? this.locality.getCountry().getDisplay() + " " : "");
		}
		return address.toString().trim();
	}

	/**
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object object) {
		if (!(object instanceof MailAddress)){
			return false;
        }
		MailAddress rhs = (MailAddress) object;
		return new EqualsBuilder()
		.append(this.getStreet(), rhs.getStreet())
		.append(this.getAt(), rhs.getAt())
		.append(this.getCivic(), rhs.getCivic())
		.append(this.getHamlet(), rhs.getHamlet())
		.append(this.getLocality(), rhs.getLocality())
		.append(this.getZipCode(), rhs.getZipCode())
		.isEquals();
		
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		HashCodeBuilder hb = new HashCodeBuilder(1816801317, 257614045)
		.append(this.at)
		.append(this.civic)
		.append(this.hamlet)
		.append(this.locality)
		.append(this.street)
		.append(this.zipCode);
		if(super.getEntity()!=null)
			hb.append(super.getEntity().getId());
		return hb.toHashCode();
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		ToStringBuilder sb = new ToStringBuilder(this,ToStringStyle.SHORT_PREFIX_STYLE);
		sb.append("at",this.at)
		.append("civic",this.civic)
		.append("hamlet",this.hamlet)
		.append("locality",this.locality)
		.append("street",this.street)
		.append("zipCode",this.zipCode);
		return sb.toString();
	}
	
	
}