/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
package net.smartlab.web.registry;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

import net.smartlab.web.Enumeration;

/**
 * Represents a fully qualified MessangerAddress usable for archive addressee
 *  
 * 
 * @author rlogiacco
 * @hibernate.joined-subclass schema="registry" table="`messenger-address`"
 * @hibernate.joined-subclass-key column="`id`"
 */
public class MessengerAddress extends Address {

	private final static long serialVersionUID = 5917703384188478000L;

	/**
	 * List of available protocols
	 */
	private final static Protocol[] PROTOCOL_LIST = new Protocol[] {Protocol.YAHOO, Protocol.MSN, Protocol.ICQ,
			Protocol.AOL, Protocol.JABBER};

	/**
	 * Used protocol on a messanger..it is
	 */
	private Protocol protocol;

	/**
	 * The unique id
	 */
	private String uid;

	
	/**
	 * Default Constructor
	 */
	public MessengerAddress() {
		super();
	}


	/**
	 * Constructor using fields
	 * 
	 * @param uid The uid to be set
	 * @param protocol The protocol type to be set
	 */
	public MessengerAddress(String uid,Protocol protocol) {
		super();
		this.protocol = protocol;
		this.uid = uid;
	}
	/**
	 * Constructor using fields
	 * 
	 * @param uid The uid to be set
	 * @param protocolCode The protocol code to be decoded
	 * @throws IllegalArgumentException for a not valid protocol code.
	 */
	public MessengerAddress(String uid,int protocolCode) {
		super();
		this.protocol = (Protocol)new Protocol().decode(protocolCode);
		this.uid = uid;
	}

	/**
	 * Returns the uid
	 * 
	 * @hibernate.property column="`uid`" length="100"
	 * @return returns the uid.
	 */
	public String getUid() {
		return uid;
	}

	/**
	 * Sets the uid
	 * 
	 * @param uid the uid to set.
	 */
	public void setUid(String uid) {
		this.uid = uid;
	}

	/**
	 * Return protocols
	 * 
	 * @return returns the <code>protocol</code>.
	 * @hibernate.property column="`protocol`"
	 *                     type="net.smartlab.web.registry.MessengerAddress$Protocol"
	 */
	public Protocol getProtocol() {
		return protocol;
	}

	/**
	 * Returns an array of all available protocols 
	 * 
	 * @return returns the <code>Protocol</code> list.
	 */
	public Protocol[] getProtocols() {
		return PROTOCOL_LIST;
	}

	/**
	 * Sets the protocol
	 * 
	 * @param protocol the <code>protocol</code> to set.
	 */
	public void setProtocol(Protocol protocol) {
		this.protocol = protocol;
	}

	/**
	 * @see net.smartlab.web.registry.Address#getFormattedString()
	 */
	public String getFormattedString() {
		return (protocol !=null ? protocol.getDisplay() : "N/A") + ":" + uid;
	}


	/**
	 * In computing, a protocol is a convention or standard that controls or 
	 * enables the connection, communication, and data transfer between two 
	 * computing endpoints. </br>
	 * In its simplest form, a protocol can be defined as the rules governing the syntax, 
	 * semantics, and synchronization of communication. 
	 * In this simple case it is represented by an <code>Enumeration</code>
	 * Object to associate an appropriate simple display/code to a <code>MessangerAddress</code>
	 * 
	 * @author rlogiacco
	 * @author gcontartese
	 */
	public static class Protocol extends Enumeration {

		private static final long serialVersionUID = 8233518844869946720L;

		/**
		 * Identifies the standard 'Yahoo' 
		 */
		public final static Protocol YAHOO = new Protocol('Y', "yahoo");

		/**
		 * Identifies the standard 'Msn'
		 */
		public final static Protocol MSN = new Protocol('M', "msn");

		/**
		 * Identifies the standard 'Icq'
		 */
		public final static Protocol ICQ = new Protocol('I', "icq");

		/**
		 * Identifies the standard 'Aol'
		 */
		public final static Protocol AOL = new Protocol('A', "aol");

		/**
		 * Identifies the standard 'Jabber'
		 */
		public final static Protocol JABBER = new Protocol('J', "jabber");

		/**
	     * <p><code>Protocol</code> instances should NOT be constructed in
	     * standard programming. Instead, the class should be used as
	     * <code>Protocol."protocolType"</code>.
	     * Es.: Protocol.MSN
	     * </p>
	     *
	     * <p>This constructor is public to permit tools that require a JavaBean
	     * instance to operate.</p>
	     */
		public Protocol() {
			super();
		}

		/**
		 * Constructor using field
		 * 
		 * @param id The id to be set
		 * @param display the display to be set
		 */
		private Protocol(int id, String display) {
			super(id, display);
		}

		/**
		 * @see net.smartlab.web.Enumeration#decode(int)
		 */
		public Enumeration decode(int id) {
			switch (id) {
				case 'Y':
					return YAHOO;
				case 'M':
					return MSN;
				case 'I':
					return ICQ;
				case 'A':
					return AOL;
				case 'J':
					return JABBER;
				default:
					throw new IllegalArgumentException("Not valid protocol id");
			}
		}
	}
	
	/**
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object object) {
		if (!(object instanceof MessengerAddress)){
			return false;
        }
		MessengerAddress rhs = (MessengerAddress) object;
		return new EqualsBuilder()
		.append(this.uid, rhs.getUid())
		.append(this.protocol, rhs.getProtocol())
		.isEquals();
		
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return new HashCodeBuilder(1816801317, 257614045)
		.append(this.uid)
		.append(this.protocol)
		.toHashCode();
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		ToStringBuilder sb = new ToStringBuilder(this,ToStringStyle.SHORT_PREFIX_STYLE);
		sb.append("Uid",this.uid)
		.append("Procol",this.protocol!=null ?this.protocol.getDisplay() : "N/A");
		return sb.toString();
	}
}