/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
 
package net.smartlab.web.registry;

import java.sql.Date;

import net.smartlab.web.Enumeration;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;



/**
 * Represents a phisical person registry entry.
 * 
 * @see Entity
 * @author rlogiacco
 * @author gcontartese
 * 
 * @hibernate.joined-subclass schema="registry" table="`person`"
 * @hibernate.joined-subclass-key column="`id`"
 */
public class Person extends Entity {

    private final static long serialVersionUID = 6755075929706100845L;

    /**
     * Available genders.
     */
    public final static Gender[] GENDER_LIST = new Gender[] {
            Gender.UNKNOWN, Gender.MALE, Gender.FEMALE
    };

    /**
     * Describe the full name of a person .It is a component 
     * @clientCardinality 1
     * @supplierCardinality 0..1
     * @uml.property name="name"
	 * @uml.associationEnd aggregation="composite" inverse="person:net.smartlab.web.registry.Name"
	 * @uml.association name="has a"
     */
    private Name name = new Name();
    /**
     * Defualt gender 
     */
    private Gender gender = Gender.UNKNOWN;

    private Date birthdate;

    /**
     * The birthplace locality
     * @see Locality
     * @see MailAddress
     * @clientCardinality 1
     * @supplierCardinality 0..1
     * @uml.property  name="birthplace"
     * @uml.associationEnd  inverse="person:net.smartlab.web.registry.Locality"
     */

    private Locality birthplace = new Locality();

    /** 
	 * Describe the residence as a component for the person.
	 * @see MailAddress
	 * @see Locality
	 * @directed true
	 * @uml.property name="residence"
	 * @uml.associationEnd aggregation="shared" inverse="person:net.smartlab.web.registry.MailAddress"
	 */
    private MailAddress residence;
    /**
     * It must be unique and used to further search info on Person.
     * In the most of countries it is used as String and generated by 
     * person properties. 
     */
    private String idCode;
    /**
     * flag to describe if the person agree about a privacy contract or not.
     */
    private boolean privacy;


    /**
     * Default constructor
     */
    public Person(){
    	super();
    }
    
    /**
	 * Simple constructor using name
	 * @param name
	 */
	public Person(Name name) {
		super();
		this.name = name;
	}
	/**
	 * Constructor using fields
	 * @param name The anme to be set
	 * @param gender the gender to be set
	 */
	public Person(Name name, Gender gender) {
		super();
		this.name = name;
		this.gender = gender;
	}
	/**
	 * Constructor using fields
	 * @param name the name to be set
	 * @param gender the gender to be set
	 * @param birthdate The bithdate to be set
	 */
	public Person(Name name, Gender gender, Date birthdate) {
		super();
		this.name = name;
		this.gender = gender;
		this.birthdate = birthdate;
	}

	/**
     * Return the full name of a person as a component 
     * @hibernate.component
     * @return returns the name.
     */
    public Name getName() {
        return name;
    }

    /**
     * Setter of the component name. 
     * @param name the name to set.
     */
    public void setName(Name name) {
        this.name = name;
    }

    /**
     * UNKNOWN as default it returns the person gender
     * @hibernate.property column="`gender`"
     *                     type="net.smartlab.web.registry.Person$Gender"
     * @return Returns the gender.
     * @see Gender
     */
    public Gender getGender() {
        return gender;
    }

    /**
     * Returns a list of available genders.
     * 
     * @return a list of available genders.
     */
    public Gender[] getGenders() {
        return GENDER_LIST;
    }

    /**
     * @param gender The gender to set.
     */
    public void setGender(Gender gender) {
        this.gender = gender;
    }

    /**
     * Return the birthdate 
     * @hibernate.property column="`birthdate`"
     * @return returns the birthdate.
     */
    public Date getBirthdate() {
        return birthdate;
    }

    /**
     * Setter the birthdate property.
     * @param birthdate the birthdate to set.
     */
    public void setBirthdate(Date birthdate) {
        this.birthdate = birthdate;
    }

    /**
     * Return the locality of birthplace.
     * @hibernate.component
     * @return returns the birthplace.
     */
    public Locality getBirthplace() {
        return birthplace;
    }

    /**
     * Sets the birthplace 
     * @param birthplace the birthplace to set.
     */
    public void setBirthplace(Locality birthplace) {
        this.birthplace = birthplace;
    }

    /** 
	 * Return the mail address of the <code>Person</code> as a component 
	 * @hibernate.many-to-one column="`residence`" cascade = "all" inverse="true"
	 * @return returns the residence.
	 * @uml.property  name="residence"
	 */
    public MailAddress getResidence() 
	{
		return residence;
	}

    /** 
	 * Sets the residence place with the given MailAdrress
	 * @param residence the residence to set.
	 * @uml.property  name="residence"
	 */
    public void setResidence(MailAddress residence) 
	{
		this.residence = residence;
	}

    /**
     * Return the idCode
     * @return returns the idCode.
     * @hibernate.property column="`idCode`"
     */
    public String getIdCode() {
        return idCode;
    }

    /**
     * Sets the idCode.
     * @param idCode the idCode to set.
     */
    public void setIdCode(String idCode) {
        this.idCode = idCode;
    }
    /**
     * Describe a flag for each case of  property
     * @return returns the <code>privacy</code>.
     * @hibernate.property column="`privacy`"
     */
    public boolean isPrivacy() {
        return privacy;
    }

    /**
     * Sets the privacy flag .
     * @param privacy the <code>privacy</code> to set.
     */
    public void setPrivacy(boolean privacy) {
        this.privacy = privacy;
    }
    /**
	 * This class represents a <code>Gender</code> type of a person.
	 * You have to use it to complete, in a regular way,the person description.
	 * <p>
	 * All the known gender are described below as a static Enumeration:
	 * <ul>
	 * <li><tt>MALE</tt></li>
	 * <li><tt>FEMALE</tt></li>
	 * <li><tt>UNKNOWN</tt></li>
	 * </ul> 
	 * As a standard <code>Enumeration</code> behavior you can use it to set gender
	 * property of the Person in a static way or to obtain it from your custom code
	 * by <tt>decode</tt> method.
	 * .<br>
	 * For Example :
	 * <blockquote><pre>
	 * 			//...Setting person gender 
	 * 			Person person=new Person();
	 *			person.setGender(Person.Gender.FEMALE);
	 *			//..Retrieving an instance by a simple code
	 *			..... 
	 *			int code=....;
	 *			Person.Gender gender=new Person.Gender();
	 *			person.setGender(gender.decode(code);
	 *</pre></blockquote>
	 * @see net.smartlab.web.Enumeration
	 * @author rlogiacco
	 * @author gcontartese
	 */
    public static class Gender extends Enumeration {

        private static final long serialVersionUID = -2404603797399493275L;

        /**
         * Represents the MALE natural gender
         */
        public final static Gender MALE = new Gender('M', "gender.male");

        /**
         * Represents the FEMALE natural gender
         */
        public final static Gender FEMALE = new Gender('F', "gender.female");

        /**
         * Represents the UNKNOWN natural gender
         */
        public final static Gender UNKNOWN = new Gender(0, "gender.unknown");


        /**
         * Default constructor.
         */
        public Gender() {
            super();
        }

        /**
         * Constructor by id and display.
         * @param id
         * @param display
         */
        private Gender(int id, String display) {
            super(id, display);
        }

        /**
         * @see net.smartlab.web.Enumeration#decode(int)
         */
        public Enumeration decode(int id) {
            switch (id) {
                case 'M':
                    return MALE;
                case 'F':
                    return FEMALE;
                default:
                    return UNKNOWN;
            }
        }
    }
    public boolean equals(Object object) {
		if (!(object instanceof Person)){
			return false;
        }
		Person rhs = (Person) object;
		return new EqualsBuilder()
		.append(this.getBirthdate(), rhs.getBirthdate())
		.append(this.getBirthplace(), rhs.getBirthplace())
		.append(this.getGender(), rhs.getGender())
		.append(this.getIdCode(), rhs.getIdCode())
		.append(this.getName(), rhs.getName())
		//.append(this.getResidence(), rhs.getResidence())
		.isEquals();
		
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return new HashCodeBuilder(220707833, 1784534049)
		.append(this.birthdate)
		.append(this.birthplace)
		.append(this.gender)
		.append(this.idCode)
		.append(this.name)
		//.append(this.residence)
		.toHashCode();
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return new ToStringBuilder(this,ToStringStyle.SHORT_PREFIX_STYLE)
		.append(this.name)
		.append("birthdate",this.birthdate)
		.append(this.birthplace)
		.append(this.gender.getDisplay())
		.append("idCode",this.idCode)
		//.append("Residence",this.residence)
		.toString();
	}

	/**
	 * @see net.smartlab.web.registry.Entry#compareTo(java.lang.Object)
	 */
	public int compareTo(Object entry) {
		if(entry instanceof Person){
			if(super.compareTo(entry) == 0){
				Person personToCompare = (Person) entry;
				return this.toString().compareTo(personToCompare.toString());
			}
			return super.compareTo(entry);
		}
		throw new ClassCastException();
	}


    
}
