/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */

package net.smartlab.web.registry;
import java.util.Collection;
import java.util.List;

import net.smartlab.web.BusinessObjectFactory;
import net.smartlab.web.DAOException;
import net.smartlab.web.DataAccessObject;
import net.smartlab.web.page.CollectionPaginator;

import org.hibernate.Criteria;
import org.hibernate.FetchMode;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Projections;

/**
 * The PersonFactory class implements a list of reach methods to 
 * retrieve more informations about the <code>Person</code>.
 * Like any other BusinessObjectFactory it contains basic methods to   
 * access and retrieve information on the specified Person
 * Object. 
 * Each person is strongly linked to its list of belonging. 
 * Consequently follows is implemented methods to access a specified 
 * group of person in the given list.
 * Each of these methods is characterized by a SearchInfo to filter, 
 * if it is deemed necessary, on the properties of the table Person.
 * Assuming you want to get a Collection of Person contained in a list 
 * specified and whose lastname begins with "c" :
 * <blockquote><pre>
 *  idList=//...retrieving idList
 *  SearchInfo info=new SearchInfo();
 *  info.addFilter("name.lastname",SearchInfo.LIKE,"c%");
 * 	PresonFactory factory=PersonFactory.getInstance();
 *  Collection result=factory.listByList(idList,info);
 * </pre></blockquote>
 * That's all you have to do.
 * 
 * @author rlogiacco
 * @author gcontartese
 * @see Person
 * @see net.smartlab.web.BusinessObjectFactory
 * @see net.smartlab.web.DataAccessObject.SearchInfo
 */
public class PersonFactory extends BusinessObjectFactory {

	/**
	 * Singleton instance statically initialized.
	 */
	private final static PersonFactory instance = new PersonFactory();

	/**
	 * Blocks class instantiation to enforce the singleton pattern.
	 */
	private PersonFactory() {
		super();
	}

	/**
	 * @see net.smartlab.web.BusinessObjectFactory#getMappedClass()
	 */
	public Class getMappedClass() {
		return Person.class;
	}

	/**
	 * @return Returns the singleton instance.
	 */
	public static PersonFactory getInstance() {
		return instance;
	}
	/**
	 * 
	 * Page a collection of distinct Person order by lastname,firstname properties.
	 * Each element belong to the specified given <code>List</code>
	 * @param listId .Id of belonging list.
	 * @param lastname. Filter property computing in a like expression.
	 * @param firstname. Filter property computing in a like expression.
	 * @return Page a collection of distinct Person order by lastname,firstname properties.
	 * @throws DAOException .if occurs any problem during transaction.
	 * @deprecated see pageByList(long idList,SearchInfo info).
	 */
	public Collection pageByListLastnameFirstname(long listId, String lastname, String firstname) throws DAOException {
		try {
			if (logger.isDebugEnabled()) {
				logger.debug("listByListLastnameFirstname() - start");
			}
			Session session = super.current();
			String hqlQuery = "select distinct person from Person person left join fetch person.addresses a join person.lists parent where parent.id=" + listId;
			if (lastname != null && lastname.trim().length() > 0) {
				hqlQuery = hqlQuery + " and UPPER(person.name.lastname) like '" + lastname.toUpperCase() + "%'";
			}
			if (firstname != null && firstname.trim().length() > 0) {
				hqlQuery = hqlQuery + " and UPPER(person.name.firstname) like '" + firstname.toUpperCase() + "%'";
			}
			hqlQuery = hqlQuery + " order by person.name.lastname,person.name.firstname";
			Query query = session.createQuery(hqlQuery);
			return new CollectionPaginator(query.list());
		} catch (Exception e) {
			throw new DAOException("listByListLastnameFirstname()", e);
		}
	}
	/**
	 * Return a <code>Collection</code> of distinct persons elements which 
	 * are included in the specified <code>List</code> list.
	 * The result could be filtered by <code>SearchInfo</code> info. 
	 * A way to use this method could be :<br>
	 * <blockquote><pre>
	 * 		
	 * 		long idList=//retrieve idList...
	 * 		SearchInfo info=new SearchInfo();
	 *  	info.addFilter("name.lastname",SearchInfo.LIKE,StringUtils.upperCase("c%"));
	 *  	Collection result=pageByList(idList,info).
	 * </pre></blockquote>
	 * 
	 * @param listId The id of list wich contains the elements
	 * @param searchInfo. The <code>SearchInfo</code> to filter the elements. 
	 * @return A Collection paginated of persons filtered by searchinfo filters.
	 * Note:SearchInfo could be empty but not <code>null</code>
	 * @throws 	DAOException . if occurs problems during transaction. 
	 * @throws  NullPointerException if SearchInfo is null
	 * @see	    DataAccessObject.SearchInfo
	 * @see     List
	 * @see     Entity
	 */
	public Collection listByList (long listId, SearchInfo searchInfo) throws DAOException {
		try {
			if (logger.isDebugEnabled()) {
				logger.debug("pageByList() START");
			}
			Criteria criteria = super.createCriteria(searchInfo);
			if (listId > 0) {
				criteria.createAlias("lists", "list");
				criteria.add(Expression.eq("list.id", new Long(listId)));
				criteria.setResultTransformer(Criteria.DISTINCT_ROOT_ENTITY);
			}
			return criteria.list();

		} catch (HibernateException he) {
			logger.error("listByListLastnameFirstname() ERROR :" + he.getMessage());
			throw new DAOException("listByListLastnameFirstname() ERROR :" + he.getMessage());
		} catch (Exception e) {
			e.printStackTrace();
			throw new DAOException("listByListLastnameFirstname() ERROR :" + e.getMessage());
		}
	}

	/**
	 * Return a <code>Collection</code> of distinct persons elements which 
	 * are included in the specified <code>List</code> list.
	 * The result could be filtered by <code>SearchInfo</code> info. 
	 * A way to use this method could be :<br>
	 * <blockquote><pre>
	 * 		
	 * 		long idList=//retrieve idList...
	 * 		SearchInfo info=new SearchInfo();
	 *  	info.addFilter("name.lastname",SearchInfo.LIKE,StringUtils.upperCase("c%"));
	 *  	Collection result=pageByList(idList,info).
	 * </pre></blockquote>
	 * 
	 * @param listId The id of list wich contains the elements
	 * @param searchInfo. The <code>SearchInfo</code> to filter the elements. 
	 * @return A Collection paginated of persons filtered by searchinfo filters.
	 * Note: could note be <code>null</code>
	 * @throws 	DAOException . if occurs problemd during transaction. 
	 * @throws  NullPointerException if SearchInfo is null
	 * @see	    DataAccessObject.SearchInfo
	 * @see     List
	 * @see     Entity
	 */
	public Collection pageByList(long listId, SearchInfo searchInfo) throws DAOException {
		try {
			
			if (logger.isDebugEnabled()) {
				logger.debug("pageByList() START");
			}
			Criteria criteria = super.createCriteria(searchInfo);
			if (listId > 0) {
				criteria.createAlias("lists", "list");
				criteria.add(Expression.eq("list.id", new Long(listId)));
				criteria.setResultTransformer(Criteria.DISTINCT_ROOT_ENTITY);
			}
			return new Paginator(criteria);

		} catch (HibernateException he) {
			logger.error("listByListLastnameFirstname() ERROR :" + he.getMessage());
			throw new DAOException("listByListLastnameFirstname() ERROR :" + he.getMessage());
		} catch (Exception e) {
			e.printStackTrace();
			throw new DAOException("listByListLastnameFirstname() ERROR :" + e.getMessage());
		}
	}
	/**
	 * 
	 * Return a not paginated <code>Collection</code> of <code>Person</code> elements contained in the first passthrought
	 * <b>list</b> excluding all the elements presents in the other list <b>notInListId</b>.
	 * @param listId. Id of principal list
	 * @param notInListId. Id of secondary list  
	 * @param info. SearchInfo further filter on person properties.	
	 * @return A Collection of <code>Person</code> excluding all the persons found
	 * in the other list.Note the SearchInfo could be empty but not null.
	 * @throws NullPointerException if the SearchInfo object is null.
	 * @throws DAOException if errors occurring during transaction.
	 * @see net.smartlab.web.registry.List
	 * @see net.smartlab.web.registry.Entity
	 * @see net.smartlab.web.DaoAccessObject.SearchInfo
	 */
	public Collection listByNotInList(long listId,long notInListId,SearchInfo info) throws DAOException {
		try {
			if (logger.isDebugEnabled()) {
				logger.debug("pageByNotInLists() - start");
			}
			Criteria personInListCriteria=super.current().createCriteria(this.getMappedClass());
			personInListCriteria.createAlias("lists", "lists");
			personInListCriteria.add(Expression.eq("lists.id", new Long(notInListId)));
			personInListCriteria.setProjection(Projections.property("id"));
			Collection results=personInListCriteria.list();
			Criteria criteria=createCriteria(info);
			criteria.createAlias("lists", "parent");
			criteria.add(Expression.eq("parent.id", new Long(listId)));
			criteria.add(Expression.not(Expression.in("id", results)));
			criteria.setFetchMode("addresses", FetchMode.JOIN);
			criteria.setResultTransformer(Criteria.DISTINCT_ROOT_ENTITY);
			return criteria.list();
			
		}catch(Exception e){
			throw new DAOException("pageByNotInLists()" + e.getMessage());
		}
		
	}	
	/**
	 * 
	 * Return a paginated <code>Collection</code> of <code>Person</code> elements contained in the first passthrought
	 * <b>list</b> excluding all the elements presents in the other list <b>notInListId</b>
	 * @param listId. Id of principal list
	 * @param notInListId. Id of secondary list  
	 * @param info. SearchInfo further filter on person properties.	
	 * @return A Collection of <code>Person</code> excluding all the persons found
	 * in the other list.Note the SearchInfo could be empty but not null.
	 * @throws NullPointerException if the SearchInfo object is null.
	 * @throws DAOException if errors occurring during transaction.
	 * @see net.smartlab.web.registry.List
	 * @see net.smartlab.web.registry.Entity
	 * @see net.smartlab.web.DaoAccessObject.SearchInfo
	  */
	public Collection pageByNotInList(long listId,long notInListId,SearchInfo info) throws DAOException {
		try {
			if (logger.isDebugEnabled()) {
				logger.debug("pageByNotInLists() - start");
			}
			Criteria personInListCriteria=super.current().createCriteria(this.getMappedClass());
			personInListCriteria.createAlias("lists", "lists");
			personInListCriteria.add(Expression.eq("lists.id", new Long(notInListId)));
			personInListCriteria.setProjection(Projections.property("id"));
			Collection results=personInListCriteria.list();
			Criteria criteria=createCriteria(info);
			criteria.createAlias("lists", "parent");
			criteria.add(Expression.eq("parent.id", new Long(listId)));
			criteria.add(Expression.not(Expression.in("id", results)));
			criteria.setFetchMode("addresses", FetchMode.JOIN);
			criteria.setResultTransformer(Criteria.DISTINCT_ROOT_ENTITY);
			return new Paginator(criteria);
			
		}catch(Exception e){
			throw new DAOException("pageByNotInLists()" + e.getMessage());
		}
		
	}
	
	
}