/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
 
package net.smartlab.web.registry;

import net.smartlab.web.Enumeration;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

/**
 * Represents a concrete class of address which describe all kind of
 * known phones .
 * In help to specify a peculiar type of the PhoneAddress use its owner inner class
 * <code>Type</code>.
 * <p>
 * Below it described the list of all <code>PhoneAddress.Type</code> known:
 * <ul>
 * <li><tt>MOBILE</tt></li>
 * <li><tt>HOME</tt></li>
 * <li><tt>FAX</tt></li>
 * <li><tt>OFFICE</tt></li>
 * <li><tt>PAGER</tt></li>
 *</ul> 
 * @author rlogiacco
 * @hibernate.joined-subclass schema="registry" table="`phone-address`"
 * @hibernate.joined-subclass-key column="`id`"
 */
public class PhoneAddress extends Address {

	private final static long serialVersionUID = 3869122455081575457L;

	/**
	 * Static list containing all the known <code>PhoneAddress.Type</code>
	 * types.
	 */
	public final static Type[] TYPE_LIST = new Type[] {
			Type.MOBILE, 
			Type.HOME, 
			Type.OFFICE, 
			Type.PAGER, 
			Type.FAX};

	/**
	 * It describes an hypothetical physical location of the PhoneAddress.
	 */
	private MailAddress location;
	
	/**
	 * Represents the logical number of the phone.
	 */
	private String number;
	/**
	 * Characterizes the type of the <tt>PhoneAddress</tt>
	 */
	private Type type;

	/**
	 * Default constructor
	 */
	public PhoneAddress() {
		super();
	}
	
	/**
	 * Constructor using fields
	 * @param number The display number
	 * @param type The type to be set
	 */
	public PhoneAddress(String number, Type type) {
		super();
		this.number = number;
		this.type = type;
	}
	/**
	 * Constructor using fields
	 * @param number The display number
	 * @param typeCode The type to be set
	 * @throws IllegalArgumentException for a not valid typeCode
	 */
	public PhoneAddress(String number, int typeCode) {
		super();
		this.number = number;
		this.type = (Type)(new Type().decode(typeCode));
	}
	/**
	 * Constructor using fields
	 * @param number The display number
	 * @param type The type to be set
	 * @param location The location to be set
	 */
	public PhoneAddress(String number, Type type, MailAddress location) {
		super();
		setLocation(location);
		this.number = number;
		this.type = type;
	}

	/**
	 * Returns the hypothetical location of the address.
	 * @return the location
	 * @hibernate.many-to-one column="`location`" cascade = "all" inverse ="true"
	 */
	public MailAddress getLocation() {
		return location;
	}

	/**
	 * Set the hypothetical location of the address.
	 * @param location. The address to set.
	 */
	public void setLocation(MailAddress location) {
		if(location != null && getEntity() != null)
			location.setEntity(getEntity());
		this.location = location;
	}

	/**
	 * Returns the number  
	 * @hibernate.property column="`number`" length="20"
	 * @return Returns the number.
	 */
	public String getNumber() {
		return number;
	}

	/**
	 * Setter of the property number
	 * @param number The number to set.
	 */
	public void setNumber(String number) {
		this.number = number;
	}

	/**
	 * Getter of property type
	 * @hibernate.property column="`type`"
	 *                     type="net.smartlab.web.registry.PhoneAddress$Type"
	 * @return Returns the type.
	 */
	public Type getType() {
		return type;
	}

	/**
	 * Return all the known types.
	 * @return
	 */
	public Type[] getTypes() {
		return TYPE_LIST;
	}

	/**
	 * Setter of property type
	 * @param type The type to set.
	 */
	public void setType(Type type) {
		this.type = type;
	}
    
    /**
     * @see net.smartlab.web.registry.Address#getFormattedString()
     */
    public String getFormattedString(){
        return number;
    }

    /**
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object object) {
		if (!(object instanceof PhoneAddress)){
			return false;
        }
		PhoneAddress rhs = (PhoneAddress) object;
		return new EqualsBuilder()
		.append(this.number, rhs.getNumber())
		.append(this.type, rhs.getType())
		.isEquals();
		
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return new HashCodeBuilder(1816801317, 257614045)
		.append(this.number)
		.append(this.type)
		.toHashCode();
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return new ToStringBuilder(this,ToStringStyle.SHORT_PREFIX_STYLE)
		.append("Number",this.number)
		.append("Type",this.type!=null ? this.type.getDisplay() :"N/A")
		.toString();
	}

	/**
	 * @see net.smartlab.web.registry.Address#compareTo(java.lang.Object)
	 */
	public int compareTo(Object object) {
		PhoneAddress newPhone = (PhoneAddress) object;
		if(super.compareTo(newPhone) == 0){
			int cmp = normalizeNull(getNumber()).compareTo(normalizeNull(newPhone.getNumber()));
			if(cmp == 0){
				return this.toString().compareTo(newPhone.toString());
			}
			return cmp;
		}
		return super.compareTo(newPhone);
	}
	
	/**
	 * This class represents a custom known phone address type.
	 * You have to use it to complete, in a regular way,the phone description.
	 * <p>
	 * All the known types are described below as a static Enumeration:
	 * <ul>
	 * <li><tt>MOBILE</tt></li>
	 * <li><tt>HOME</tt></li>
	 * <li><tt>FAX</tt></li>
	 * <li><tt>OFFICE</tt></li>
	 * <li><tt>PAGER</tt></li>
	 * </ul> 
	 * As a standard <code>Enumeration</code> behavior you can use it to set type
	 * property of the PhoneAddress in a static way or to obtain it from your custom code
	 * by <tt>decode</tt> method.
	 * .<br>
	 * For Example :
	 * <blockquote><pre>
	 * 			//...Setting address type 
	 * 			PhoneAddress phone=new PhoneAddress();
	 *			phone.setType(PhoneAddress.Type.MOBILE);
	 *			//..Retrieving an instance by a simple code
	 *			..... 
	 *			int code=....;
	 *			PhoneAddress.Type type=new PhoneAddress.Type();
	 *			phone.setType(type.decode(code);
	 *</pre></blockquote>
	 * @see net.smartlab.web.Enumeration
	 * @author rlogiacco
	 * @author gcontartese
	 */
	public static class Type extends Enumeration {

		private static final long serialVersionUID = -5554496741521318547L;
		
		/**
		 * describes a Mobile phone.
		 */
		public final static Type MOBILE = new Type('M', "phone.mobile");
		
		/**
		 * describes a Home phone.
		 */
		public final static Type HOME = new Type('H', "phone.home");
		
		/**
		 * describes a Office phone.
		 */
		public final static Type OFFICE = new Type('O', "phone.office");
		
		/**
		 * describes a Pager phone.
		 */
		public final static Type PAGER = new Type('P', "phone.pager");
		
		/**
		 * describes a Fax phone.
		 */
		public final static Type FAX = new Type('F', "phone.fax");

		
		public Type() {
			super();
		}
		/* Common constructor to define a new Enumeration 
		 * from id and display properties.
		 */
		private Type(int id, String display) {
			super(id, display);
		}

		/**
		 * @see net.smartlab.web.Enumeration#decode(int)
		 */
		public Enumeration decode(int id) {
			switch (id) {
				case 'M':
					return MOBILE;
				case 'H':
					return HOME;
				case 'O':
					return OFFICE;
				case 'P':
					return PAGER;
				case 'F':
					return FAX;
				default:
					throw new IllegalArgumentException ("Not a valid id TYPE ");
			}
		}
	}
}