/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
package net.smartlab.web.registry;

import net.smartlab.web.BusinessObject;

/**
 * Represents a relationship between two registry entities. Relationships are
 * defined as a name describing the relationship type, a relationship source and
 * a relationship target. A parent-child relationship could be represented as a
 * <i>parent </i> relationship originating from the parent and directed to the
 * child. The relationship could be navigated inverted searching for any child
 * having the desired <i>parent</i>. If a relationship doesn't have a preferred
 * direction, like a neighbors, than two separate copies of the relationship
 * should be created to ensure full browseability.
 * 
 * @author rlogiacco
 * @stereotype component
 */
public class Relationship extends BusinessObject {

	private static final long serialVersionUID = -6755482377883259670L;

	/**
	 * The relationship target.
	 * 
	 * @directed true
	 */
	private Entity target;

	/**
	 * A name identifying the relationship.
	 */
	private String name;


	/**
	 * Default empty constructor.
	 */
	protected Relationship() {
		super();
	}

	/**
	 * Instantiates a relationship between two registry items.
	 * 
	 * @param target the ending entity.
	 * @param name the relationship type.
	 */
	public Relationship(Entity target, String name) {
		this.target = target;
		this.name = name;
	}

	/**
	 * Returns the relationship name.
	 * 
	 * @return returns the name.
	 * @hibernate.property column="`name`" length="250" not-null="true"
	 */
	public String getName() {
		return name;
	}

	/**
	 * Sets the relationship name.
	 * 
	 * @param name the name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Returns the item to which the relationship is directed.
	 * 
	 * @return returns the target.
	 * @hibernate.many-to-one column="`target`" not-null="true"
	 */
	public Entity getTarget() {
		return target;
	}

	/**
	 * Sets the item to which the relationship is directed.
	 * 
	 * @param target the target to set.
	 */
	public void setTarget(Entity target) {
		this.target = target;
	}

	/**
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj) {
		if (obj instanceof Relationship) {
			Relationship relationship = (Relationship)obj;
			return relationship.target.getId() == this.target.getId() && relationship.name.equals(this.name);
		}
		return false;
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return (this.name + this.target.getId()).hashCode();
	}
}
